/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_FLDR_HPP_
#define _HEAVY_CONTEXT_WSTD_FLDR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlCast.h"
#include "HvSignalEnvelope.h"
#include "HvControlBinop.h"
#include "HvMath.h"
#include "HvSignalLine.h"
#include "HvSignalVar.h"
#include "HvControlIf.h"
#include "HvControlVar.h"

class Heavy_WSTD_FLDR : public HeavyContext {

 public:
  Heavy_WSTD_FLDR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_FLDR();

  const char *getName() override { return "WSTD_FLDR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        FLDR = 0xE42C892E, // Fldr
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_DTTaXsrE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_mcvWuw4I_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_4C5451gU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1LUeYyOV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_OtUMzdqt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_mgQK3rCn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Z94sG6ao_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_CKpWi2DC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_QNg6SVkg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_5fLS25lb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_NkqyIzWb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_6CGd3yVh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_GCk506zZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_Rkj8QtTp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_iR14Jlrj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_WG7LaB2I_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_ahL9untj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_iqAOWahH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_MM9u5T6x_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Orxa71c7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_jjhcnDR4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_5xS4OjwV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_MNQxknES_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Bbv0v8EE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ltXfc0HQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_42osALE0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_yE3XIEXv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_a3lLBzEI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Mng5wQoh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_fXvC4Z3e_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_X6u0Rvl3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_1zBF9pHR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_h8GWtBZJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_ebQW6F8u_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_kK5zzhG0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_rrxlHycj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_4efev184_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_DpNnfTOX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_KW7YdZ5B_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_uioZYBKo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ma78RYyt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_dCgwwQYF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_8Iy7Cqw6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_IYltq7Zd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_MW6jJipR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_lAajVpEO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_J4mVnoFp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_RjQxRBxE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_h63sHvxw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_9OpZvyXV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_67ndEYhl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Z5mihje1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_MxeREZn4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_At4Jy33b_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_TrcV7AwW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Liwm4QkQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_R0GAQ20e_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_dC6I7hnT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_gAPmsb5X_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_O1ri90CX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_TPk0MODA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_aifHwLmA_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_ZciDtAm1;
  SignalEnvelope sEnv_iqAOWahH;
  SignalEnvelope sEnv_MM9u5T6x;
  SignalEnvelope sEnv_kK5zzhG0;
  SignalEnvelope sEnv_rrxlHycj;
  SignalLine sLine_zlN7qlD7;
  SignalLine sLine_tUAI3FGN;
  ControlBinop cBinop_DTTaXsrE;
  ControlBinop cBinop_mcvWuw4I;
  ControlBinop cBinop_4C5451gU;
  ControlBinop cBinop_1LUeYyOV;
  ControlBinop cBinop_Z94sG6ao;
  ControlBinop cBinop_QNg6SVkg;
  ControlBinop cBinop_5fLS25lb;
  ControlBinop cBinop_NkqyIzWb;
  ControlVar cVar_6CGd3yVh;
  ControlVar cVar_Rkj8QtTp;
  SignalVarf sVarf_PxkuVRSz;
  ControlVar cVar_iR14Jlrj;
  ControlIf cIf_WG7LaB2I;
  ControlIf cIf_ahL9untj;
  ControlBinop cBinop_Orxa71c7;
  ControlBinop cBinop_5xS4OjwV;
  ControlBinop cBinop_MNQxknES;
  ControlBinop cBinop_42osALE0;
  ControlBinop cBinop_a3lLBzEI;
  SignalVarf sVarf_KzXvwtgh;
  ControlVar cVar_1zBF9pHR;
  ControlIf cIf_h8GWtBZJ;
  ControlIf cIf_ebQW6F8u;
  ControlBinop cBinop_4efev184;
  ControlBinop cBinop_KW7YdZ5B;
  ControlBinop cBinop_uioZYBKo;
  ControlBinop cBinop_8Iy7Cqw6;
  ControlBinop cBinop_MW6jJipR;
  ControlVar cVar_h63sHvxw;
  ControlBinop cBinop_9OpZvyXV;
  SignalVarf sVarf_QuUIlnbT;
  SignalVarf sVarf_dtGyk3in;
  SignalVarf sVarf_AykbViYZ;
  SignalVarf sVarf_1C9TEfwO;
  ControlBinop cBinop_At4Jy33b;
};

#endif // _HEAVY_CONTEXT_WSTD_FLDR_HPP_
